<?php

require_once(dirname(__FILE__).'/pclzip/pclzip.lib.php');

function download($theme_name, $user_theme_name, $includeThemler = true) {
    //register_shutdown_function('fatal_error_shutdown_handler');

    $theme_dir = get_theme_dir($theme_name);
    if (!file_exists($theme_dir)){
        die('Error : No Theme Folder');
    }

    $useBaseTheme = empty($user_theme_name) || ($user_theme_name == $theme_name);
    if ($useBaseTheme) {
        $user_theme_name = $theme_name;
        $replace_data = null;
        $preview_replace_data = null;
    }
    else {
        $replace_data = array('method' => array('FilesHelper', 'replace_theme_name'), 'old_name' => $theme_name, 'new_name' => $user_theme_name);
        $preview_replace_data = array('method' => array('FilesHelper', 'replace_theme_name'), 'old_name' => get_preview_theme_name($theme_name), 'new_name' => get_preview_theme_name($user_theme_name));
    }

    $uniqid = uniqid(time());
    $base_upload_dir = get_themler_storage_dir() . '/' . $uniqid;
    $user_template_dir = create_temp_theme($base_upload_dir, $theme_dir, $user_theme_name, $replace_data);

    $archive_file = $base_upload_dir . '/theme_' . $uniqid .  '.zip';
    $archive = new PclZip($archive_file);
 
    if ($includeThemler) {
        $base_preview_dir = get_preview_theme_dir($theme_dir);
        $preview_name = get_preview_theme_name($user_theme_name);
        $preview_theme_dir = create_temp_theme($base_upload_dir, $base_preview_dir, $preview_name, $preview_replace_data);

        add_theme($archive, $base_upload_dir, $preview_theme_dir, $preview_name);
        add_theme($archive, $base_upload_dir, $user_template_dir, $user_theme_name);
        add_modules($archive, $base_upload_dir); 
    } else {
        clearDirs($user_template_dir, array('export', 'project', 'themler'));
        add_theme($archive, $base_upload_dir, $user_template_dir, $user_theme_name);
    }

    add_files_to_archive_root($archive, $user_template_dir);

    if (is_readable($archive_file)) {
        $archive_content = array(
            'ext' => 'zip',
            'name' => $user_theme_name . '.zip',
            'content' => FilesHelper::read_file($archive_file)
        );

        FilesHelper::remove_dir($base_upload_dir, true);
        return $archive_content;
    }

    die('Archive file: ' . $archive_file. ' is not readable');
}

function create_temp_theme($base_upload_dir, $base_theme_dir, $theme_name, $replace_data) {
    $theme_dir = $base_upload_dir . '/' . $theme_name;
    FilesHelper::create_dir($theme_dir);
    FilesHelper::copy_recursive($base_theme_dir, $theme_dir, null, $replace_data);
    return $theme_dir; 
}

function add_theme(&$archive, $base_upload_dir, $theme_dir, $theme_name) {
    $versions     = get_support_versions();
    foreach ($versions as $version) {
        $base_dir   = $base_upload_dir . (has_current_version($version) ? '' : '/' . $version) . '/themes';
        $upload_dir    = $base_dir . '/' . $theme_name;
        $version_files = $upload_dir . '/versions/' . $version;

        FilesHelper::create_dir($upload_dir);
        FilesHelper::copy_recursive($theme_dir, $upload_dir);
        FilesHelper::copy_file($version_files . '/'. $theme_name . INF_EXT, $upload_dir . '/'. $theme_name . INFO_EXT);
        add_sub_archive($archive, $base_upload_dir, $upload_dir, $base_dir, $theme_name, $version);
    }
}

function add_sub_archive(&$archive, $base_upload_dir, $upload_dir, $removed_archive_dir, $name, $version) {
    $archive_name = $name.'-'.$version .  '.zip';
    $archive_file = $base_upload_dir . '/' . $archive_name;
    $sub_archive = new PclZip($archive_file);
    add_to_archive($sub_archive, $upload_dir, $removed_archive_dir);
    add_to_archive($archive, $archive_file, $base_upload_dir);
}

function add_modules(&$archive, $base_upload_dir) {
    $name       = 'themler';
    $base_dir   = $base_upload_dir . '/modules';
    $upload_dir = $base_dir . '/' . $name;

    FilesHelper::create_dir($upload_dir);
    FilesHelper::copy_recursive(get_modules_dir() . '/' . $name, $upload_dir);
    add_sub_archive($archive, $base_upload_dir, $upload_dir, $base_dir, $name, get_cms_version());
}

function clearDirs($base_path, $dirs, $self = true) {
    foreach ($dirs as $dir) {
        FilesHelper::remove_dir($base_path . '/' . $dir, $self);
    }
}

function delete_from_archive(&$archive, $fileName, $from_root_only = false) {
    if (($list = $archive->listContent()) == 0) {
        die("Error : ".$zip->errorInfo(true));
    }

    $delete_str = '';
    for ($i = 0; $i < sizeof($list); $i++) {
        if (($from_root_only && substr($list[$i]['filename'], 0, strlen($fileName)) === $fileName)
            || (!$from_root_only && strpos($list[$i]['filename'], $fileName) !== FALSE)) {
            $delete_str .= ($delete_str ? ',' : '') . "$i";
        }
    }

    if ($delete_str) {
        $v_list = $archive->delete(PCLZIP_OPT_BY_INDEX, $delete_str);
        if ($v_list == 0) {
            die("Error : ".$archive->errorInfo(true));
        }
    }
}

function add_to_archive(&$archive, $dir, $remove_path) {
    $v_list = $archive->add($dir, PCLZIP_OPT_ADD_PATH, '', PCLZIP_OPT_REMOVE_PATH, $remove_path);
    if ($v_list == 0) {
        die('Error to add ' . $dir . ': ' . $archive->errorInfo(true));
    }
}

function add_files_to_archive_root(&$archive, $user_template_dir) {
    $paths = array('ReadMe.txt' => FilesHelper::normalize_path($user_template_dir . '/ReadMe.txt'));
    foreach ($paths as $name => $path) {
        if (!file_exists($path))
            continue;

        add_to_archive($archive, $path, $user_template_dir);
    }
}

function zip_extract($from_file, $to_dir) {
    if (!file_exists($to_dir))
        FilesHelper::create_dir($to_dir, 0777);
    if (class_exists('ZipArchive', false)) {
        $zip = new ZipArchive();
        if ($zip->open($from_file) === true && $zip->extractTo($to_dir) && $zip->close())
            return true;
        return false;
    }
    else {
        $zip = new PclZip($from_file);
        $list = $zip->extract(PCLZIP_OPT_PATH, $to_dir, PCLZIP_OPT_REPLACE_NEWER);
        foreach ($list as $file)
            if ($file['status'] != 'ok' && $file['status'] != 'already_a_directory')
                return false;
        return true;
    }
}